use std::collections::HashMap;

use logos::{Lexer, Logos};

use crate::error::{DiagnosticKind, DiagnosticReporter, FileIdent};

#[derive(Logos, Debug, PartialEq)]
pub enum Token {
    #[regex("[a-zA-Z\\d\\s]+", |lex| lex.slice().to_owned())]
    Text(String),
    #[token("(")]
    ParenOpen,
    #[token(")")]
    ParenClose,
    #[regex(r#""([^"\\]|\\["\\bnfrt]|u[a-fA-F0-9]{4})*""#, |lex| lex.slice().to_owned())]
    String(String),
    #[token("=")]
    Equals,
    #[token("*")]
    Asterisk,
    #[token("_")]
    Underscore,
    #[token("\\")]
    Backslash,
    #[token("~")]
    WavyThing,
    #[token("✨")]
    Sparkles,
    #[token("|| ")]
    HeadingLevelIndicator,
    #[token("\n")]
    Newline,
}

#[derive(Debug)]
pub enum LanguageStructureThingy {
    Text(String),
    Sexpression {
        name: String,
        attrs: HashMap<String, String>,
        content: Box<LanguageStructureThingy>,
    },
}

pub fn parse_the_shit_out_of_this_but_manually<'a>(
    text: &str,
    reporter: &mut DiagnosticReporter<'a>,
    file_id: &'a FileIdent,
) -> Vec<LanguageStructureThingy> {
    let mut lex = Token::lexer(text).spanned();
    let mut r = Vec::new();

    while let Some((res, loc)) = lex.next() {
        match res {
            Ok(Token::Text(s)) => r.push(LanguageStructureThingy::Text(s)),
            Ok(Token::Newline) => r.push(LanguageStructureThingy::Text("\n".to_owned())),
            Ok(Token::ParenOpen) => hehe_sexpression_funy(&mut r, &mut lex),
            Err(())
            | Ok(Token::ParenClose)
            | Ok(Token::String(_))
            | Ok(Token::Equals)
            | Ok(Token::Asterisk)
            | Ok(Token::Underscore)
            | Ok(Token::Backslash)
            | Ok(Token::WavyThing)
            | Ok(Token::Sparkles)
            | Ok(Token::HeadingLevelIndicator) => {
                reporter.report_diagnostic(*file_id, loc, DiagnosticKind::InvalidToken)
            }
        }
    }

    reporter.fail_step_if_failed();

    r
}

fn hehe_sexpression_funy(r: &mut Vec<LanguageStructureThingy>, lex: &mut Lexer<'_, Token>) {
    if let Some(Ok(Token::Text(s))) = lex.next() {
        let strs = s.split_whitespace().collect::<Vec<&str>>();

        let name = strs[0].to_owned();
        let mut attrs = HashMap::new();

        if strs.len() == 1 {
        } else if strs.len() == 2 && (lex.next() == Some(Ok(Token::Equals))) {
            if let Some(Ok(Token::Text(next))) = lex.next() {
                let _ = attrs.insert(strs[1].to_owned(), next.trim_end().to_owned());
            }
        } else {
            todo!()
        }

        let content = if let Some(Ok(Token::String(s))) = lex.next() {
            Box::new(LanguageStructureThingy::Text(s))
        } else {
            todo!()
        };

        r.push(LanguageStructureThingy::Sexpression {
            name,
            attrs,
            content,
        });
        if let Some(Ok(Token::ParenClose)) = lex.next() {
            return;
        }
        // handle other things too
        todo!()
    }
}

fn mk_simple_sexpression(name: impl ToString, content: impl ToString) -> LanguageStructureThingy {
    LanguageStructureThingy::Sexpression {
        name: name.to_string(),
        attrs: HashMap::new(),
        content: Box::new(LanguageStructureThingy::Text(content.to_string())),
    }
}
